using UnityEngine;
using System.Collections;


public struct autofitData 
{
    public Vector3 center;
    public float height;
    public float radius;
    public Vector3 angles;
}

public class AutofitSingle
{
    int[] idxH0;
    int[] idxR0;
    int[] idxH1;
    int[] idxR1;
    // string name;
    bool useradiusforcenter;

    public AutofitSingle(int[] h0, int[] h1, int[] r0, int[] r1, bool radc = true)
    {
        idxH0 = h0;
        idxH1 = h1;
        idxR0 = r0;
        idxR1 = r1;
        useradiusforcenter = radc;
    }

    public autofitData calculate(Vector3[] list, Transform anchor)
    {
        Vector3 h0 = new Vector3(0,0,0);
        Vector3 h1 = new Vector3(0,0,0);
        Vector3 r0 = new Vector3(0,0,0);
        Vector3 r1 = new Vector3(0,0,0);
        foreach(int h in idxH0)
        {
            h0 += list[h];
        }
        h0 /= idxH0.Length;
        foreach(int h in idxH1)
        {
            h1 += list[h];
        }
        h1 /= idxH1.Length;
        foreach(int r in idxR0)
        {
            r0 += list[r];
        }
        r0 /= idxR0.Length;
        foreach(int r in idxR1)
        {
            r1 += list[r];
        }
        r1 /= idxR1.Length;

        autofitData ret;
        ret.center = (idxR0.Length>0 && idxR1.Length>0 && useradiusforcenter==true) ? (h0 + h1 + r0 + r1) / 4f : (h0 + h1) / 2f;
        ret.height = (h1-h0).magnitude;
        ret.radius = (idxR0.Length>0 && idxR1.Length>0) ? (r1-r0).magnitude/2f : idxR0.Length>0 ? (r0-ret.center).magnitude : ret.height/2.0f;
        Vector3 euler = anchor.transform.InverseTransformDirection(h1-h0);
        Quaternion rot = Quaternion.FromToRotation(Vector3.up, euler);
        ret.angles =  rot.eulerAngles;

        return ret;
    }


}

public class AutofitInstructions
{
    static public readonly Hashtable instructions = new Hashtable() 
    {
        { "lshin0", new AutofitSingle( new int[]{8510}, new int[]{2626}, new int[]{47}, new int[]{8698} ) },
        { "lshin1", new AutofitSingle( new int[]{8501}, new int[]{4101}, new int[]{474,484}, new int[]{446} ) },
        { "lshin2", new AutofitSingle( new int[]{495}, new int[]{4097}, new int[]{487}, new int[]{448,6153} ) },
        { "lshin3", new AutofitSingle( new int[]{9320,9304}, new int[]{9332}, new int[]{9335,9336}, new int[]{10342,9326} ) },
        { "lshin4", new AutofitSingle( new int[]{498}, new int[]{512,509}, new int[]{525}, new int[]{501} ) },
        { "lshin5", new AutofitSingle( new int[]{543}, new int[]{519,4112}, new int[]{529}, new int[]{506} ) },
        { "lshin6", new AutofitSingle( new int[]{539,2804}, new int[]{4116,4117}, new int[]{534,533}, new int[]{560,562} ) },
        { "lshin7", new AutofitSingle( new int[]{2804,8543}, new int[]{4118,4120}, new int[]{557,8626}, new int[]{36,26} ) },
        { "lshin8", new AutofitSingle( new int[]{8544}, new int[]{4114,4113}, new int[]{34}, new int[]{8630} ) },

        { "lthigh0", new AutofitSingle( new int[]{8510}, new int[]{2626}, new int[]{47}, new int[]{8698} ) },
        { "lthigh1", new AutofitSingle( new int[]{6697,8475}, new int[]{2397}, new int[]{383}, new int[]{8784} ) },
        { "lthigh2", new AutofitSingle( new int[]{6668,6670}, new int[]{399,402,403}, new int[]{381,382}, new int[]{2365,2363} ) },
        { "lthigh3", new AutofitSingle( new int[]{6683,373}, new int[]{6079}, new int[]{6085,367}, new int[]{6151,6152} ) },
        { "lthigh4", new AutofitSingle( new int[]{8523}, new int[]{10355,2396}, new int[]{2339}, new int[]{6108,6123} ) },
        { "lthigh5", new AutofitSingle( new int[]{8210}, new int[]{7882,7858}, new int[]{7841,7844}, new int[]{} ) },
        { "lthigh6", new AutofitSingle( new int[]{7782}, new int[]{2391}, new int[]{298,299}, new int[]{} ) },

        { "lpelvis0", new AutofitSingle( new int[]{7782}, new int[]{2391}, new int[]{298,299}, new int[]{} ) },
        
        // first configuration
        // { "lpelvis1", new AutofitSingle( new int[]{8752}, new int[]{13603,13605,13213,11193}, new int[]{2388,2389}, new int[]{8666} ) },
        // { "lpelvis2", new AutofitSingle( new int[]{22664,323,22891}, new int[]{8991,8992,8993,2945,2946}, new int[]{22653,22657}, new int[]{2384,274} ) },

        // { "labdomenA0", new AutofitSingle( new int[]{2955}, new int[]{18814,13740}, new int[]{7144,7138}, new int[]{8995,8997} ) },
        // { "labdomenA1", new AutofitSingle( new int[]{10565,10439,10865,10866}, new int[]{4036,4037,8663,8666}, new int[]{8997,8994}, new int[]{2952,2821} ) },

        // { "labdomenB0", new AutofitSingle( new int[]{10498,10497,10529}, new int[]{2834,2867,2901}, new int[]{10545,10567,10870}, new int[]{21180,21177,13685} ) },
        // { "labdomenB1", new AutofitSingle( new int[]{10649,10653}, new int[]{13297,13286,21213,21077}, new int[]{2465,2794}, new int[]{10602,10601} ) },

        // better configuration?
        { "mhip0", new AutofitSingle( new int[]{300,8819}, new int[]{11228,19490}, new int[]{2118,2117}, new int[]{2097,2093} ) },
        { "mhip1", new AutofitSingle( new int[]{2921,2938}, new int[]{2778}, new int[]{2118}, new int[]{2097,2096} ) },

        { "mabdomenA0", new AutofitSingle( new int[]{2859,2858}, new int[]{13661,13660}, new int[]{2831}, new int[]{2850} ) },

        { "mabdomenB0", new AutofitSingle( new int[]{10399}, new int[]{21057}, new int[]{2088}, new int[]{10814} ) },
        { "mabdomenB1", new AutofitSingle( new int[]{10785,10779}, new int[]{21430,21436}, new int[]{10560}, new int[]{2100,2461} ) },
        { "mabdomenB2", new AutofitSingle( new int[]{10819,2467}, new int[]{13603,2797,280,11208}, new int[]{8152}, new int[]{2944} ) },

        { "lbreast0", new AutofitSingle( new int[]{0}, new int[]{10894,7130}, new int[]{8,2404}, new int[]{2591}, false )  },

    };

    static AutofitInstructions()
    {

    }

}

