using System;
using UnityEngine;
using UnityEngine.UI;
using UnityEngine.Events;
using System.Text;
using System.Collections;
using System.Collections.Generic;
using MVR.FileManagementSecure;
using SimpleJSON;
using GPUTools.Physics.Scripts.Behaviours;

public class FakeCollider 
    {
        Atom atom;
        CapsuleCollider ccollider;
        CapsuleLineSphereCollider cgpu;
        public DAZBone anchor;
        GameObject preview;
        Vector3 center;
        Vector3 rotation;
        JSONStorableFloat radius;
        JSONStorableFloat height;
        JSONStorableFloat x;
        JSONStorableFloat y;
        JSONStorableFloat z;
        JSONStorableFloat rx;
        JSONStorableFloat ry;
        JSONStorableFloat rz;
        UIDynamicSlider s_radius;
        UIDynamicSlider s_height;
        UIDynamicSlider s_x;
        UIDynamicSlider s_y;
        UIDynamicSlider s_z;
        UIDynamicSlider s_rx;
        UIDynamicSlider s_ry;
        UIDynamicSlider s_rz;


        bool showpreview = false;
        bool gui = false;
        public bool seethrough=true;
        // bool issphere;
        public string name;
        
        
        public FakeCollider(MVRScript scr, string objid, string anchorid, string n)// bool sphere=false) 
        {
            name = n;
            atom = scr.GetAtomById(objid);
            atom.mainController.physicsEnabled = false;
            atom.collisionEnabledJSON.val = false;//false;
            atom.hidden = true;
            center = new Vector3(0,0,0);
            rotation = new Vector3(0,0,0);

            Renderer[] rs = atom.gameObject.GetComponentsInChildren<Renderer>();
            foreach (var r in rs) 
            {
                r.enabled = false;
            }
            DelayedMeshRenderer[] rs2 = atom.gameObject.GetComponentsInChildren<DelayedMeshRenderer>();
            foreach (var r in rs2) 
            {
                r.enabled = false;
            }


            Atom p = scr.GetContainingAtom();
            JSONStorable _anchor = p.GetStorableByID(anchorid);
            anchor = _anchor as DAZBone;

            ccollider = atom.GetComponentInChildren<CapsuleCollider>();
            cgpu = atom.GetComponentInChildren<CapsuleLineSphereCollider>();
            cgpu.capsuleCollider = ccollider;
            cgpu.UpdateData();


            initSliders();

        }

        public void initSliders()
        {
            radius = new JSONStorableFloat("radius",ccollider.radius, (val) => 
                { 
                    ccollider.radius=val;
                    cgpu.UpdateData();
                    UpdatePreview();
                },
                0, 0.1f , false);
            height = new JSONStorableFloat("height",ccollider.height, (val) => 
                { 
                    ccollider.height=val;
                    cgpu.UpdateData();
                    UpdatePreview();
                },
                0, 0.3f, false);
            x = new JSONStorableFloat("x",ccollider.center.x, (val) => 
                { 
                    center.x = val;
                    cgpu.UpdateData();
                    UpdatePreview();
                },
                -0.1f, 0.1f, false);
            y = new JSONStorableFloat("y",ccollider.center.y, (val) => 
                { 
                    center.y = val;
                    cgpu.UpdateData();
                    UpdatePreview();
                },
                -0.7f, 0.1f, false);
            z = new JSONStorableFloat("z",ccollider.center.z, (val) => 
                { 
                    center.z = val;
                    cgpu.UpdateData();
                    UpdatePreview();
                },
                -0.1f, 0.1f, false);
                
            rx = new JSONStorableFloat("rx", rotation.x, (val) => 
                {
                    rotation.x=val;
                    ccollider.transform.localEulerAngles = rotation;
                    cgpu.UpdateData();
                    UpdatePreview();
                },
                0f, 360f, true);                    
            ry = new JSONStorableFloat("ry",rotation.y, (val) => 
                { 
                    rotation.y=val;
                    ccollider.transform.localEulerAngles = rotation;
                    cgpu.UpdateData();
                    UpdatePreview();
                },
                0f, 360f, true); 
            rz = new JSONStorableFloat("rz",rotation.z, (val) => 
                { 
                    rotation.z=val;
                    ccollider.transform.localEulerAngles = rotation;
                    cgpu.UpdateData();
                    UpdatePreview();
                },
                0f, 360f, true);                     
    
        }

        public void createPreview()
        {
            showpreview=true;
            preview=GameObject.CreatePrimitive(PrimitiveType.Capsule);
            preview.transform.SetParent(ccollider.transform, false);

            foreach (var c in preview.GetComponents<Collider>())
            {
                c.enabled = false;
                UnityEngine.Object.Destroy(c);
            }

            setInactive();
            UpdatePreview();

        }
        public void destroyPreview()
        {
            if (showpreview) 
            {
                UnityEngine.Object.Destroy(preview);
                showpreview = false;
            }


        }

        public void setInactive() 
        {
            if (showpreview) {
                var previewRenderer = preview.GetComponent<Renderer>();
                var material = previewRenderer.material;
                var color = Color.green;//previewRenderer.material.color;
                color.a = seethrough ? 0.05f : 0.3f;
                previewRenderer.material.color = color;
            }
        }
        public void setActive() 
        {
            if (showpreview) {
                var previewRenderer = preview.GetComponent<Renderer>();
                var material = previewRenderer.material;
                var color = Color.magenta;//previewRenderer.material.color;
                color.a = seethrough ? 0.53f : 0.67f;
                previewRenderer.material.color = color;
            }
        }

        public void createGUI(MVRScript scr)
        {
            s_radius = scr.CreateSlider(radius);
            s_height = scr.CreateSlider(height);
            s_x = scr.CreateSlider(x);
            s_y = scr.CreateSlider(y);
            s_z = scr.CreateSlider(z);
            s_radius.valueFormat="F8";
            s_height.valueFormat="F8";
            s_x.valueFormat="F8";
            s_y.valueFormat="F8";
            s_z.valueFormat="F8";
            s_rx = scr.CreateSlider(rx,true);
            s_ry = scr.CreateSlider(ry,true);
            s_rz = scr.CreateSlider(rz,true);

            setActive();
            gui = true;
        }

        public void destroyGUI(MVRScript scr)
        {
            if (!gui) return;
            scr.RemoveSlider(s_radius);
            scr.RemoveSlider(s_height);
            scr.RemoveSlider(s_x);
            scr.RemoveSlider(s_y);
            scr.RemoveSlider(s_z);
            scr.RemoveSlider(s_rx);
            scr.RemoveSlider(s_ry);
            scr.RemoveSlider(s_rz);
            setInactive();
            gui = false;

        }

        public void UpdateTransform() //redundant?
        {

            ccollider.radius = radius.val;
            ccollider.height = height.val;
            Vector3 _angles = new Vector3(rx.val,ry.val,rz.val);
            ccollider.transform.localEulerAngles=_angles;
            cgpu.UpdateData();

            UpdatePreview();
        }

        public void mirrorData(FakeCollider source) 
        {
            SetTransform(source.radius.val, source.height.val, -source.x.val, source.y.val, source.z.val, source.rx.val, 360-source.ry.val, 360-source.rz.val);
        }

        public void SetTransform(float rad, float h, float xx, float yy, float zz, float rxx, float ryy, float rzz)
        {
            radius.val = rad;
            height.val = h;
            x.val = xx;
            y.val = yy;
            z.val = zz;
            rx.val = rxx;
            ry.val = ryy;
            rz.val = rzz;

        }

        public void Autofit(AutofitSingle instr, Vector3[] verts)
        {
            autofitData data = instr.calculate(verts, anchor.transform);
            Vector3 pos = data.center;
            float r = data.radius;
            float h = data.height;
            Vector3 rot = data.angles;
            pos = anchor.transform.InverseTransformPoint(pos);
            float offset =  0.06f;
            float addh = 0f;
            SetTransform(r, h+addh, pos.x, pos.y-offset, pos.z, rot.x, rot.y, rot.z);
        }

        public void UpdatePreview()
        {
            if (showpreview) {
                var previewRenderer = preview.GetComponent<Renderer>();
                var material = previewRenderer.material;
                if (seethrough)
                {
                    material.shader = Shader.Find("Battlehub/RTGizmos/Handles");
                    material.SetFloat("_Offset", 1f);
                    material.SetFloat("_MinAlpha", 1f);
                    previewRenderer.material = material;
                }
                else
                {
                    material.shader = Shader.Find("Standard");
                    material.SetInt("_SrcBlend", (int)UnityEngine.Rendering.BlendMode.SrcAlpha);
                    material.SetInt("_DstBlend", (int)UnityEngine.Rendering.BlendMode.OneMinusSrcAlpha);
                    material.SetInt("_ZWrite", 0);
                    material.DisableKeyword("_ALPHATEST_ON");
                    material.EnableKeyword("_ALPHABLEND_ON");
                    material.DisableKeyword("_ALPHAPREMULTIPLY_ON");
                    material.renderQueue = 3000;
                    previewRenderer.material = material;
                }

                float size = ccollider.radius * 2;
                float height = ccollider.height / 2;
                preview.transform.localScale = new Vector3(size, height, size);
                if (ccollider.direction == 0)
                    preview.transform.localRotation = Quaternion.AngleAxis(90, Vector3.forward);
                else if (ccollider.direction == 2)
                    preview.transform.localRotation = Quaternion.AngleAxis(90, Vector3.right);

            }
        }

        public void Update()
        {   
            atom.mainController.transform.SetPositionAndRotation(anchor.transform.TransformPoint(center), anchor.transform.rotation);
        }

        public void Destroy()
        {
            destroyPreview();
        }

        public JSONClass GetJSON()
        {
            JSONClass jc = new JSONClass();
            jc["radius"].AsFloat = radius.val;
            jc["height"].AsFloat = height.val;
            jc["x"].AsFloat = x.val;
            jc["y"].AsFloat = y.val;
            jc["z"].AsFloat = z.val;
            jc["rx"].AsFloat = rx.val;
            jc["ry"].AsFloat = ry.val;
            jc["rz"].AsFloat = rz.val;
            return jc;
        }

        public void RestoreFromJSON(JSONClass jc)
        {
            radius.val = jc["radius"].AsFloat;
            height.val = jc["height"].AsFloat;
            x.val = jc["x"].AsFloat;
            y.val = jc["y"].AsFloat;
            z.val = jc["z"].AsFloat;
            rx.val = jc["rx"].AsFloat;
            ry.val = jc["ry"].AsFloat;
            rz.val = jc["rz"].AsFloat;
            // UpdateTransform();
        }
        
    }