using System;
using UnityEngine;
using UnityEngine.UI;
using System.Collections;
using System.Collections.Generic;
using MVR.FileManagementSecure;
using SimpleJSON;

public class UYCol : MVRScript
{
    List<FakeCollider> fakecolliders;

    JSONStorableStringChooser fakecolliderchooser;
    JSONClass manualJC;
    string _savePath = "Saves\\UYFakeColliders";
    string _saveExt = "uy";

    UIDynamicButton buttonLR;
    UIDynamicButton buttonRL;
    UIDynamicButton buttonFit;
    bool LRconfirm;
    bool RLconfirm;
    bool fitconfirm;

    List<string> atomNames;
    List<string> fcolNames;
    // List<Atom> atoms;
    private bool atoms_created=false;
    private bool is_loading=true;
    private bool was_loading=true;

    private IEnumerator createCapsule(string atomname, string anchorname, string fcolname)
    {
        if (GetAtomById(atomname)==null) {
            yield return SuperController.singleton.AddAtomByType("Capsule", atomname);
        }
            // atoms.Add(GetAtomById(atomname));
        atomNames.Add(atomname);
        fakecolliders.Add(new FakeCollider(this,atomname, anchorname, fcolname));
        fcolNames.Add(fcolname);
    }


    private IEnumerator createAtom(string atype, string aid) {
        if (GetAtomById(aid)==null) {
            yield return SuperController.singleton.AddAtomByType(atype,aid);
        }
    }

    private IEnumerator createAllAtoms() 
    {
        //left 
        for (int i=0; i<9; i++)
        {
            yield return createCapsule($"uycol_lshin{i}", "lShin", $"lshin{i}");
        }
        for (int i=0; i<7; i++)
        {
            yield return createCapsule($"uycol_lthigh{i}", "lThigh", $"lthigh{i}");
        }
        for (int i=0; i<1; i++)
        {
            yield return createCapsule($"uycol_lpelvis{i}", "pelvis", $"lpelvis{i}");
        }

        for (int i=0; i<1; i++)
        {
            yield return createCapsule($"uycol_lbreast{i}", "lPectoral", $"lbreast{i}");
        }


        //middle
        for (int i=0; i<2; i++)
        {
            yield return createCapsule($"uycol_mhip{i}", "hip", $"mhip{i}");
        }
        for (int i=0; i<1; i++)
        {
            yield return createCapsule($"uycol_mabdomenA{i}", "abdomen", $"mabdomenA{i}");
        }
        for (int i=0; i<3; i++)
        {
            yield return createCapsule($"uycol_mabdomenB{i}", "abdomen2", $"mabdomenB{i}");
        }



        //right
        for (int i=0; i<9; i++)
        {
            yield return createCapsule($"uycol_rshin{i}", "rShin", $"rshin{i}");
        }
        for (int i=0; i<7; i++)
        {
            yield return createCapsule($"uycol_rthigh{i}", "rThigh", $"rthigh{i}");
        }
        for (int i=0; i<1; i++)
        {
            yield return createCapsule($"uycol_rpelvis{i}", "pelvis", $"rpelvis{i}");
        }
        for (int i=0; i<1; i++)
        {
            yield return createCapsule($"uycol_rbreast{i}", "rPectoral", $"rbreast{i}");
        }

        lateInit();
        manualRestore(null);
        atoms_created = true;
    }

    private void removeAtom(string aid)
    {
        Atom a = GetAtomById(aid);
        if (a!=null)
        {
            SuperController.singleton.RemoveAtom(a);
        }
    }
    private void removeAllAtoms()
    {
        foreach (string atomname in atomNames)
        {
            removeAtom(atomname);
        }
    }

    // public void testinggrounds()
    // {
    //     Atom p = GetContainingAtom();
    //     DAZSkinV2 skin = p.GetComponentInChildren<DAZCharacter>().skin;
    //     SuperController.LogMessage($"found a skin ? {skin!=null}");
    //     Vector3[] verts = new Vector3[5000];
    //     skin.smoothedVertsBuffer.GetData(verts);
    //     SuperController.LogMessage($"vertex 17 : {verts[17].x}");

    // }

 
    public override void Init()
    {
        // testinggrounds();
        atoms_created = false;
        atomNames = new List<string>();
        fcolNames = new List<string>();
        fakecolliders = new List<FakeCollider>();
    }        

    public void StartCreate()
    {
        // SuperController.LogMessage("SCENE LOADED starting to create atoms");
        StartCoroutine(createAllAtoms());
    }

    public UIDynamicPopup CreateAutoPopup(JSONStorableStringChooser jssc, bool rightSide = false)
    {
        UIDynamicPopup popup = CreateFilterablePopup(jssc, rightSide);
        popup.popup.labelText.alignment = TextAnchor.UpperCenter;
        popup.popup.labelText.GetComponent<RectTransform>().anchorMax = new Vector2(0, 0.89f);
        
        {
            var btn = Instantiate(manager.configurableButtonPrefab);
            btn.SetParent(popup.transform, false);
            Destroy(btn.GetComponent<LayoutElement>());
            btn.GetComponent<UIDynamicButton>().label = "<";
            btn.GetComponent<UIDynamicButton>().button.onClick.AddListener(() =>
            {
                popup.popup.SetPreviousValue();
            });
            var prevBtnRect = btn.GetComponent<RectTransform>();
            prevBtnRect.pivot = new Vector2(0, 0);
            prevBtnRect.anchoredPosition = new Vector2(10f, 0);
            prevBtnRect.sizeDelta = new Vector2(0f, 0f);
            prevBtnRect.offsetMin = new Vector2(5f, 5f);
            prevBtnRect.offsetMax = new Vector2(80f, 70f);
            prevBtnRect.anchorMin = new Vector2(0f, 0f);
            prevBtnRect.anchorMax = new Vector2(0f, 0f);
        }

        {
            var btn = Instantiate(manager.configurableButtonPrefab);
            btn.SetParent(popup.transform, false);
            Destroy(btn.GetComponent<LayoutElement>());
            btn.GetComponent<UIDynamicButton>().label = ">";
            btn.GetComponent<UIDynamicButton>().button.onClick.AddListener(() =>
            {
                popup.popup.SetNextValue();
            });
            var prevBtnRect = btn.GetComponent<RectTransform>();
            prevBtnRect.pivot = new Vector2(0, 0);
            prevBtnRect.anchoredPosition = new Vector2(10f, 0);
            prevBtnRect.sizeDelta = new Vector2(0f, 0f);
            prevBtnRect.offsetMin = new Vector2(82f, 5f);
            prevBtnRect.offsetMax = new Vector2(157f, 70f);
            prevBtnRect.anchorMin = new Vector2(0f, 0f);
            prevBtnRect.anchorMax = new Vector2(0f, 0f);
        }

        return popup;

    }

    public void lateInit()
    {
        JSONStorableBool toggle = new JSONStorableBool("Show Colliders", false, (val) => {
            if (val) {
                foreach( FakeCollider fcol in fakecolliders) {
                    fcol.createPreview();
                }
            } else {
                foreach( FakeCollider fcol in fakecolliders) {
                    fcol.destroyPreview();
                }
            }
            RefreshUI(fakecolliderchooser.val);

        });
        CreateToggle(toggle);

        JSONStorableBool toggle2 = new JSONStorableBool("See-through Colliders", true, (val) => {
            foreach( FakeCollider fcol in fakecolliders) {
                fcol.seethrough = val;
                fcol.UpdatePreview();
            }
            RefreshUI(fakecolliderchooser.val);

        });
        CreateToggle(toggle2);



        buttonFit = CreateButton("Guess fit", true);
        buttonFit.button.onClick.AddListener(()=> { guessFit(); });
        fitconfirm = false;

        buttonLR = CreateButton("Copy L to R",true);
        buttonLR.button.onClick.AddListener( () => {mirrorData(false);} );
        LRconfirm = false;
        buttonRL = CreateButton("Copy R to L",true);
        buttonRL.button.onClick.AddListener( () => {mirrorData(true);} );
        RLconfirm = false;

        JSONStorableString helplabel = new JSONStorableString("HLAP", 
                    "<b>Supplementary Colliders Beta :</b> Adds several hidden capsule objects to a Person, filling in areas with missing cloth and hair collisions. \n\n"
                +   "<b>Usage :</b> Use the <b>Guess Fit</b> button for an estimated fit, then adjust as needed all the colliders on one side (names start with <b>l</b>* or <b>r</b>*)"
                +   " as well as the middle colliders (<b>m</b>*), then copy to other side using the <b>L to R</b> or <b>R to L</b> buttons.\n\n"
                +   "Guessing a fit tries to position the capsules based on only a few vertices and is not exact, in particular with extreme morphs."
        );
        UIDynamic helpui = CreateTextField(helplabel, true);
        helpui.height = 457.0f;

        var loadPresetUI = CreateButton("Load Preset");
        loadPresetUI.button.onClick.AddListener(() =>
        {
            FileManagerSecure.CreateDirectory(_savePath);
            var shortcuts = FileManagerSecure.GetShortCutsForDirectory(_savePath);
            SuperController.singleton.GetMediaPathDialog(HandleLoadPreset, _saveExt, _savePath, false, true, false, null, false, shortcuts);
        });

        var savePresetUI = CreateButton("Save Preset");
        savePresetUI.button.onClick.AddListener(() =>
        {
            FileManagerSecure.CreateDirectory(_savePath);
            var fileBrowserUI = SuperController.singleton.fileBrowserUI;
            fileBrowserUI.SetTitle("Save colliders preset");
            fileBrowserUI.fileRemovePrefix = null;
            fileBrowserUI.hideExtension = false;
            fileBrowserUI.keepOpen = false;
            fileBrowserUI.fileFormat = _saveExt;
            fileBrowserUI.defaultPath = _savePath;
            fileBrowserUI.showDirs = true;
            fileBrowserUI.shortCuts = null;
            fileBrowserUI.browseVarFilesAsDirectories = false;
            fileBrowserUI.SetTextEntry(true);
            fileBrowserUI.Show(HandleSavePreset);
            fileBrowserUI.fileEntryField.text = (int)(DateTime.UtcNow - new DateTime(1970, 1, 1)).TotalSeconds + "." + _saveExt;
            fileBrowserUI.ActivateFileNameField();
        });
    

        fakecolliderchooser = new JSONStorableStringChooser("fakecolliders", fcolNames, "lshin0", "Colliders" );

        fakecolliderchooser.setCallbackFunction += RefreshUI;
        // CreateScrollablePopup(fakecolliderchooser);
        // CreateFilterablePopup(fakecolliderchooser);
        CreateAutoPopup(fakecolliderchooser);
        RefreshUI(fakecolliderchooser.val);

        try
        {
            SuperController.LogMessage($"{nameof(UYCol)} initialized");
        }
        catch (Exception e)
        {
            SuperController.LogError($"{nameof(UYCol)}.{nameof(Init)}: {e}");
        }
    }

    public void guessFit()
    {
        if (fitconfirm) {
            Atom p = GetContainingAtom();
            DAZSkinV2 skin = p.GetComponentInChildren<DAZCharacter>().skin;
            Vector3[] verts = new Vector3[25000];
            skin.smoothedVertsBuffer.GetData(verts);

            foreach (FakeCollider fcol in fakecolliders) 
            {
                if (AutofitInstructions.instructions.ContainsKey(fcol.name)) {
                    AutofitSingle instr = AutofitInstructions.instructions[fcol.name] as AutofitSingle;
                    fcol.Autofit(instr, verts);
                }            

            }
            mirrorDataInner(false);
        }
        fitconfirm = buttonState(buttonFit, !fitconfirm);
    }


    public void LateUpdate()
    {
        was_loading = is_loading;
        is_loading = SuperController.singleton.isLoading;
        if (!is_loading && was_loading) StartCreate();
        
        if (atoms_created == false) return;
        foreach( FakeCollider fcol in fakecolliders) 
        {
            fcol.Update();
        }        
    }

    public void RefreshUI(string fc)
    {
        LRconfirm = buttonState(buttonLR, false);
        RLconfirm = buttonState(buttonRL, false);
        fitconfirm = buttonState(buttonFit, false);
        foreach( FakeCollider fcol in fakecolliders) {
            fcol.destroyGUI(this);    
            if (fc == fcol.name || fc=="all") {
                fcol.createGUI(this);
            }
        }

    }

    public bool buttonState(UIDynamicButton but, bool confirm)
    {
        if (confirm)
        {
            // convert button to confirm button, return true flag 
            but.buttonColor = Color.green;
            string text = "Confirm " + but.buttonText.text;
            but.buttonText.text = text;
            return true;
        }
        else
        {
            // convert button back to regular button, return false flag 
            but.buttonColor = Color.white;
            string text = but.buttonText.text.Replace("Confirm ","");
            but.buttonText.text = text;
            return false;
        }

    }


    public void mirrorData(bool RtoL = false)
    {

        // reset both buttons
    
        // set for confirm the correct button
        if (!RtoL)
        {
            RLconfirm = buttonState(buttonRL, false);
            if (LRconfirm) {
                mirrorDataInner(RtoL);
            }
            LRconfirm = buttonState(buttonLR, !LRconfirm); 
        }
        else 
        {
            LRconfirm = buttonState(buttonLR, false);
            if (RLconfirm) {
                mirrorDataInner(RtoL);
            }
            RLconfirm = buttonState(buttonRL, !RLconfirm); 
        }
    }


    public void mirrorDataInner(bool RtoL = false) 
    {
        
        
        string sc = RtoL ? "r" : "l";
        string dc = RtoL ? "l" : "r";
    
        foreach(FakeCollider s_fcol in fakecolliders) 
        {
            if (s_fcol.name.StartsWith(sc)) 
            {
                string s_name = s_fcol.name;
                foreach(FakeCollider d_fcol in fakecolliders) 
                {

                    if (d_fcol.name.StartsWith(dc) && d_fcol.name.Substring(1)== s_fcol.name.Substring(1)) 
                    {
                        d_fcol.mirrorData(s_fcol);
                    }
                }
                
            }
        }
    }


    public JSONClass CollectJSON(JSONClass jc)
    {
        foreach( FakeCollider fcol in fakecolliders) 
        {
            jc[fcol.name] = fcol.GetJSON();

        } 
        return jc;
    }

    public override JSONClass GetJSON(bool includePhysical = true, bool includeAppearance = true, bool forceStore = false)
    {
        JSONClass jc = base.GetJSON(includePhysical, includeAppearance, forceStore);
        if (includePhysical || forceStore)
        {
            needsStore = true;
            jc = CollectJSON(jc);

        }
        return jc;
    }

    private void HandleLoadPreset(string path)
    {
        if (string.IsNullOrEmpty(path))
            return;

        manualRestore((JSONClass)LoadJSON(path));
    }
    private void HandleSavePreset(string path)
    {
        SuperController.singleton.fileBrowserUI.fileFormat = null;
        if (string.IsNullOrEmpty(path)) return;
        if (!path.ToLower().EndsWith($".{_saveExt}")) path += $".{_saveExt}";

        var presetJsonClass = new JSONClass();
        CollectJSON(presetJsonClass);
        SaveJSON(presetJsonClass, path);
    }

    public void manualRestore(JSONClass jc)
    {
        if (jc == null)  jc = manualJC;
        if (jc == null)  return;

        foreach( FakeCollider fcol in fakecolliders) 
        {
            if (jc.HasKey(fcol.name)) {
                fcol.RestoreFromJSON(jc[fcol.name].AsObject);
            }

        }

    }

    public override void LateRestoreFromJSON(JSONClass jc, bool restorePhysical = true, bool restoreAppearance = true, bool setMissingToDefault = true)
    {
        base.LateRestoreFromJSON(jc, restorePhysical, restoreAppearance, setMissingToDefault);
        if (!base.physicalLocked && restorePhysical)
        {
            manualJC = jc;
        }
    }

    public void OnEnable()
    {
        try
        {
            SuperController.LogMessage($"{nameof(UYCol)} enabled");
        }
        catch (Exception e)
        {
            SuperController.LogError($"{nameof(UYCol)}.{nameof(OnEnable)}: {e}");
        }
    }

    public void OnDisable()
    {
        try
        {
            SuperController.LogMessage($"{nameof(UYCol)} disabled");
        }
        catch (Exception e)
        {
            SuperController.LogError($"{nameof(UYCol)}.{nameof(OnDisable)}: {e}");
        }
    }

    public void OnDestroy()
   
    {
        foreach( FakeCollider fcol in fakecolliders) 
        {
            fcol.Destroy();            
        }
        removeAllAtoms();

        try
        {
            SuperController.LogMessage($"{nameof(UYCol)} destroyed");
        }
        catch (Exception e)
        {
            SuperController.LogError($"{nameof(UYCol)}.{nameof(OnDestroy)}: {e}");
        }
    }
}
